@@ -48,6 +48,7 @@
 #include <Lum/Table.h>
 #include <Lum/Text.h>
 #include <Lum/TextValue.h>
+#include <Lum/Toggle.h>
 #include <Lum/Panel.h>
 #include <Lum/Space.h>
 #include <Lum/StatusLine.h>
@@ -566,7 +567,9 @@ public:
     slider->SetModel(mines);
     label->AddLabel(L"Mines (in %):",slider);
 
-    label->AddLabel(L"One button mode:",Lum::Boolean::Create(oneButtonMode));
+    if (!Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+      label->AddLabel(L"One button mode:",Lum::Boolean::Create(oneButtonMode));
+    }
 
 
 /*
@@ -647,22 +650,27 @@ public:
 class MyWindow : public Lum::Dialog
 {
 private:
-  Sweeper               *sweeper;
-  Lum::Model::ActionRef newGameAction;
-  Lum::Model::ActionRef hiscoreAction;
-  Lum::Model::ActionRef settingsAction;
-  Lum::Model::ActionRef helpAction;
-  Lum::Model::ActionRef aboutAction;
-  Lum::Model::ActionRef timerAction;
+  Sweeper                *sweeper;
+  Lum::Model::ActionRef  newGameAction;
+  Lum::Model::BooleanRef zoom;
+  Lum::Model::ActionRef  hiscoreAction;
+  Lum::Model::ActionRef  settingsAction;
+  Lum::Model::ActionRef  helpAction;
+  Lum::Model::ActionRef  aboutAction;
+  Lum::Model::ActionRef  timerAction;
 
-  Lum::Text             *marks;
-  Lum::Text             *time;
+  Lum::Text              *marks;
+  Lum::Text              *time;
+
+  size_t                 unzoomX;
+  size_t                 unzoomY;
 
 public:
 
   MyWindow()
   : sweeper(NULL),
     newGameAction(new Lum::Model::Action),
+    zoom(new Lum::Model::Boolean(false)),
     hiscoreAction(new Lum::Model::Action),
     settingsAction(new Lum::Model::Action),
     helpAction(new Lum::Model::Action),
@@ -670,6 +678,7 @@ public:
     timerAction(new Lum::Model::Action)
   {
     Observe(newGameAction);
+    Observe(zoom);
     Observe(hiscoreAction);
     Observe(settingsAction);
     Observe(helpAction);
@@ -690,7 +699,8 @@ public:
 
     hPanel=Lum::HPanel::Create(true,false);
 
-    hPanel->Add(Lum::Button::Create(L"_New game",newGameAction));
+    hPanel->Add(Lum::Toggle::Create(L"Zoom",zoom));
+    hPanel->Add(Lum::Button::Create(L"_New",newGameAction));
     hPanel->Add(new Lum::HSpace(Lum::Space::sizeNormal,true));
 
     font=Lum::OS::display->GetFont(Lum::OS::Display::fontTypeFixed,210);
@@ -812,7 +822,7 @@ public:
         SetOneButtonMode(settings->oneButtonMode->Get());
 
         sweeper->SetOneButtonMode(settings->oneButtonMode->Get());
-        sweeper->EvaluateThemeName();
+        sweeper->SetThemeName(themeName);
         sweeper->Run();
       }
 
@@ -875,28 +885,29 @@ public:
       LoadThemes();
 
       if (!LoadConfig()) {
-        Lum::Dlg::Msg::ShowOk(this,
-                              L"No configuration file found!",
-                              L"The program was not able to find its configuration file.\n"
-                              L"\n"
-                              L"If this is the first start of your program, this is\n"
-                              L"OK. The program will generate a configuration file if\n"
-                              L"it quits.\n"
-                              L"If this is not your first use of this program, you may not\n"
-                              L"have solved any levels up to now, have not correctly quit\n"
-                              L"the program, or have deleted the history of solved levels!");
+        std::cout << "Did not find a configuration, use default..." << std::endl;
       }
 
       size_t width,height,mines;
 
       GetCurrentGame(width,height,mines);
-      sweeper->EvaluateThemeName();
+      sweeper->SetThemeName(themeName);
       sweeper->SetSize(width,height,mines);
       sweeper->SetOneButtonMode(GetOneButtonMode());
     }
     if (model==newGameAction && newGameAction->IsFinished()) {
+      sweeper->SetThemeName(themeName);
+      zoom->Set(false);
       sweeper->Run();
     }
+    else if (model==zoom && sweeper!=NULL) {
+      if (zoom->Get()) {
+        sweeper->SetThemeName(zoomThemeName);
+      }
+      else {
+        sweeper->SetThemeName(themeName);
+      }
+    }
     else if (model==hiscoreAction && hiscoreAction->IsFinished()) {
       ShowHiscore();
     }
@@ -908,17 +919,30 @@ public:
     }
     else if (model==helpAction && helpAction->IsFinished()) {
       Lum::Dlg::Help::Show(this,
-                           L"In normal mode, you just open a square by pressing\n"
-                           L"the left button on it.\n"
-                           L"You mark a square as containing a bomb by pressing\n"
-                           L"the right mouse button on it.\n"
+                           L"This application has a two button mode, for devices\n"
+                           L"that have a mouse with at least two buttons and a one\n"
+                           L"button mode in case your mouse only has one button or\n"
+                           L"you are using a touch screen (in this case you just\n"
+                           L"touch instead of click) or similar. In case your device\n"
+                           L"has two buttons, you can toggle between both modes using\n"
+                           L"a setting in the settings dialog.\n"
                            L"\n"
-                           L"If you don't have a right mouse button you can\n"
-                           L"activate the one button mouse mode in the settings\n"
-                           L"dialog.\n"
-                           L"In this case you mark squares as containing a\n"
-                           L"bomb with the left mouse button, and open a square\n"
-                           L"by doing a diagonal stroke over the square.");
+                           L"In one button mode you mark squares as containing a\n"
+                           L"bomb with a simple click, and unmark them with another\n"
+                           L"single click. You can open a square by doing a double\n"
+                           L"click. Make sure, that you do not click too fast so that\n"
+                           L"single clicks are interpreted as double clicks.\n"
+                           L"\n"
+                           L"In two button mode, you just open a square by pressing\n"
+                           L"the left button on it. You mark a square as containing\n"
+                           L"a bomb by pressing the right mouse button on it and\n"
+                           L"unmark it clicking with the right button on it again\n"
+                           L"\n"
+                           L"The game is won, if you have opened all squares that\n"
+                           L"do not contain a bomb! There is no need to mark all\n"
+                           L"squares as containg a bombto finish!\n"
+                           L"\n"
+                           L"Have fun :-)");
     }
     else if (sweeper!=NULL && model==sweeper->GetStatusModel()) {
       if (sweeper->GetStatus()==Sweeper::won) {
@@ -975,10 +999,12 @@ public:
     else if (sweeper!=NULL && model==sweeper->GetMarksModel()) {
       this->marks->SetText(Lum::Base::NumberToWString(sweeper->GetMarksModel()->Get()));
     }
-    else if (model==GetWindow()->GetMapedAction() && GetWindow()->GetMapedAction()->IsFinished()) {
+    else if (model==GetWindow()->GetMapedAction() &&
+             GetWindow()->GetMapedAction()->IsFinished()) {
       Lum::OS::display->AddTimer(1,0,timerAction);
     }
-    else if (model==GetWindow()->GetUnmapedAction() && GetWindow()->GetMapedAction()->IsFinished()) {
+    else if (model==GetWindow()->GetUnmapedAction() &&
+             GetWindow()->GetMapedAction()->IsFinished()) {
       Lum::OS::display->RemoveTimer(timerAction);
     }
 
